import sys
from typing import List
from argparse import ArgumentParser
from chroma_feedback import helper, wording
from chroma_feedback.typing import Status, Consumer
from .device import get_devices, process_devices
from .api import get_api

ARGS = None


def support() -> bool:
	return helper.is_linux() is True or helper.is_mac() is True


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--thingm-blink1-device', action = 'append')
	ARGS = helper.get_first(program.parse_known_args())


def run(status : Status) -> List[Consumer]:
	api = get_api()
	devices = get_devices(api.all_lights(), ARGS.thingm_blink1_device)

	if not devices:
		sys.exit(wording.get('device_not_found') + wording.get('exclamation_mark'))
	return process_devices(devices, status)



import sys
from typing import Any
from chroma_feedback import wording

API = None


def get_api() -> Any:
	global API

	if not API:
		API = api_factory()
	return API


def api_factory() -> Any:
	api = None

	try:
		from busylight.lights import USBLightIOError, USBLightNotFound
		from busylight.lights.luxafor import Flag

		try:
			api = Flag
			api.first_light().release()
		except (USBLightIOError, USBLightNotFound):
			sys.exit(wording.get('connection_not_found').format('LUXAFOR FLAG') + wording.get('exclamation_mark'))
		return api
	except ImportError:
		sys.exit(wording.get('package_not_found').format('BUSYLIGHT FOR HUMANS') + wording.get('exclamation_mark'))



import os
from typing import Any, Dict, List
import platform
import sys
from chroma_feedback.typing import Status, Producer


def get_producer_status(producer_result : List[Producer]) -> Status:
	status: Status = 'passed'

	# process producer

	for producer in producer_result:
		if producer['active'] is True:
			if producer['status'] == 'started':
				if status not in ['errored', 'failed']:
					status = 'started'
			if producer['status'] == 'errored':
				if status != 'failed':
					status = 'errored'
			if producer['status'] == 'failed':
				status = 'failed'
	return status


def parse_slug(slug : str) -> Dict[str, Any]:
	if slug:
		return dict(zip(['workspace', 'project'], slug.split('/')))
	return {}


def is_root() -> bool:
	return os.geteuid() == 0


def is_linux() -> bool:
	return to_lower_case(platform.system()) == 'linux'


def is_mac() -> bool:
	return to_lower_case(platform.system()) == 'darwin'


def is_windows() -> bool:
	return to_lower_case(platform.system()) == 'windows'


def to_lower_case(__string__ : Any) -> str:
	return str(__string__).lower()


def has_argument(argument : str) -> bool:
	return any(argv.startswith(argument) for argv in sys.argv)


def get_first(__list__ : Any) -> Any:
	return next(iter(__list__), None)


def get_last(__list__ : Any) -> Any:
	return next(reversed(__list__), None)



from __future__ import print_function
from typing import List
from chroma_feedback import color, helper, metadata, wording
from chroma_feedback.typing import Consumer, Producer, Report


def create_producer_report(producer_result : List[Producer]) -> List[Report]:
	report : List[Report] = []

	# process result

	for producer in producer_result:
		if producer['active'] is True:
			if producer['status'] == 'passed':
				report.append(
				{
					'status': 'passed',
					'message': wording.get('build_passed').format(producer['slug'], producer['producer']),
					'symbol': color.format_passed(wording.get('tick'))
				})
			if producer['status'] == 'started':
				report.append(
				{
					'status': 'started',
					'message': wording.get('build_started').format(producer['slug'], producer['producer']),
					'symbol': color.format_started(wording.get('hourglass'))
				})
			if producer['status'] == 'errored':
				report.append(
				{
					'status': 'errored',
					'message': wording.get('build_errored').format(producer['slug'], producer['producer']),
					'symbol': wording.get('cross')
				})
			if producer['status'] == 'failed':
				report.append(
				{
					'status': 'failed',
					'message': wording.get('build_failed').format(producer['slug'], producer['producer']),
					'symbol': color.format_failed(wording.get('cross'))
				})
	return report


def create_consumer_report(consumer_result : List[Consumer]) -> List[Report]:
	report : List[Report] = []

	# process result

	for consumer in consumer_result:
		if consumer['active'] is True:
			if consumer['status'] == 'passed':
				report.append(
				{
					'status': 'passed',
					'message': wording.get('set_passed').format(consumer['name']),
					'symbol': color.format_passed(wording.get('tick'))
				})
			if consumer['status'] == 'started':
				report.append(
				{
					'status': 'started',
					'message': wording.get('set_started').format(consumer['name']),
					'symbol': color.format_started(wording.get('hourglass'))
				})
			if consumer['status'] == 'errored':
				report.append(
				{
					'status': 'errored',
					'message': wording.get('set_errored').format(consumer['name']),
					'symbol': wording.get('cross')
				})
			if consumer['status'] == 'failed':
				report.append(
				{
					'status': 'failed',
					'message': wording.get('set_failed').format(consumer['name']),
					'symbol': color.format_failed(wording.get('cross'))
				})
	return report


def print_header() -> None:
	print(metadata.get('name') + ' ' + metadata.get('version') + ' ' + wording.get('by') + ' ' + metadata.get('author'))


def print_report(report : List[Report]) -> None:
	for value in report:
		if helper.is_windows() is False:
			print(value['symbol'] + ' ' + value['message'])
		else:
			print(value['message'])



from typing import Any, Dict, List
from argparse import ArgumentParser
import base64
from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--codeship-host', default = 'https://api.codeship.com')
		program.add_argument('--codeship-slug', action = 'append')
		program.add_argument('--codeship-username', required = True)
		program.add_argument('--codeship-password', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	if ARGS.codeship_slug:
		result.extend(fetch(ARGS.codeship_host, ARGS.codeship_slug, ARGS.codeship_username, ARGS.codeship_password))
	else:
		result.extend(fetch(ARGS.codeship_host, None, ARGS.codeship_username, ARGS.codeship_password))
	return result


def fetch(host : str, slug : str, username : str, password : str) -> List[Producer]:
	result = []
	auth = fetch_auth(host, username, password)

	if 'organizations' in auth and 'token' in auth:
		for organization in auth['organizations']:
			projects = fetch_projects(host, organization['uuid'], auth['token'])
			for project in projects:
				if not slug or project['name'] in slug:
					result.extend(fetch_builds(host, organization['uuid'], project['name'], project['uuid'], auth['token']))
	return result


def fetch_auth(host : str, username : str, password : str) -> Dict[str, Any]:
	result = {}
	response = None

	if host and username and password:
		username_password = username + ':' + password
		response = request.post(host + '/v2/auth', headers =
		{
			'Accept': 'application/json',
			'Authorization': 'Basic ' + base64.b64encode(username_password.encode('utf-8')).decode('ascii')
		})

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'access_token' in data and 'organizations' in data:
			result['token'] = data['access_token']
			result['organizations'] = data['organizations']
	return result


def fetch_projects(host : str, organization_id : str, token : str) -> List[Dict[str, Any]]:
	result = []
	response = None

	if host and organization_id and token:
		response = request.get(host + '/v2/organizations/' + organization_id + '/projects', headers =
		{
			'Accept': 'application/json',
			'Authorization': 'Bearer ' + token
		})

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'projects' in data:
			for project in data['projects']:
				result.append(project)
	return result


def fetch_builds(host : str, organization_id : str, slug : str, project_id : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and organization_id and project_id and token:
		response = request.get(host + '/v2/organizations/' + organization_id + '/projects/' + project_id + '/builds', headers =
		{
			'Accept': 'application/json',
			'Authorization': 'Bearer ' + token
		})

	# process response

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'builds' in data:
			build = helper.get_first(data['builds'])

			if build and 'status' in build:
				result.append(normalize_data(slug, build['status']))
	return result



from typing import Any, List
import copy
from chroma_feedback import color
from chroma_feedback.typing import Status, Consumer, Color
from .api import get_api


def get_lights(lights : Any, light_names : List[str]) -> Any:
	if light_names:
		for light in copy.copy(lights):
			if light.name not in light_names:
				lights.remove(light)
	return lights


def process_lights(lights : Any, status : Status) -> List[Consumer]:
	result : List[Consumer] = []

	# process lights

	for light in lights:
		result.append(
		{
			'consumer': 'philips_hue',
			'type': 'light',
			'name': light.name,
			'active': set_light(light.name, color.get_by_status(status)),
			'status': status
		})
	return result


def set_light(light_name : str, color_config : Color) -> bool:
	api = get_api(None)

	return api is not None and api.set_light(light_name,
	{
		'hue': color_config['hue'],
		'sat': color_config['saturation'][1],
		'bri': color_config['brightness'][1],
		'on': True,
		'alert': 'none'
	}) is not None
